#include <memory>
#include "intGadget.h"
#include "stringGadget.h"
#include "warning.h"
#include "colorWell.h"
#include "textItem.h"
#include "myStringDrawer.h"
#include "myPopUpMenu.h"
#include "myRadioView.h"
#include "myTextView.h"
#include "layoutMatrix.h"
#include "layoutMatrixItem.h"
#include "preferences.h"
#include "myPreferences.h"
#include "myApp.h"
#include "myColorControl.h"
#include "windowGuts.h"


//ctor--just initialize
LayoutMatrix	::	LayoutMatrix(	BWindow * pparamWindow, 
									const uint32 ui32paramFlags, 
									const uint32 ui32paramRows, 
									const uint32 ui32paramColumns) 
				: 	itemsList(new BList()), 
					mpChildrenToAdd(new BList()), 
					mpItemsToDraw(new BList()), 
					window(pparamWindow), 
					rows(ui32paramRows), 
					columns(ui32paramColumns), 
					flags(ui32paramFlags), 
					bottom(0),
					right(0)
{
//left blank
}//end


//dtor
LayoutMatrix	::	~LayoutMatrix() 
{
	delete mpChildrenToAdd;
	delete mpItemsToDraw;
}//end


//calculate the view positions of all the MatrixLayoutItems
//on the items list with offsets based upon matrixLeft
//and matrixTop
void 
LayoutMatrix	::	Calc(	const float fparamMatrixLeft,  
							const float fparamMatrixTop,
							sem_id paramSemID) 
{
	int32 numItems = itemsList->CountItems();
	if (!numItems) return;
	if (acquire_sem(paramSemID) != B_NO_ERROR)
	{
		warning("Aquire_sem failed\n");
		return;
	}
	TextItem *	scratchTextItem;
	BView view(BRect(0, 0, 32, 32), "LayoutMatrixView", B_FOLLOW_NONE, B_WILL_DRAW);
	//need to attach these items to a BView before they
	//can be Resize()ed and MoveTo()ed
	window->Lock();
	BFont * font = myPrefs->GetPreferredFont();
	for (uint32 i = 0; i < numItems; i++)
	{
		((LayoutMatrixItem *)(itemsList->ItemAt(i)))->Calc(font);
	}
	window->AddChild(&view);
	float widest = 0;
	float tallest = 0;
	uint32 index = 0;
	for (uint32 i = 0; i < numItems; i++)
	{
		LayoutMatrixItem * lmi = (LayoutMatrixItem *)itemsList->ItemAt(index++);
		if (lmi->widthPref > widest) 
		{
			widest = lmi->widthPref;
		}
		if (lmi->heightPref > tallest) 
		{
			tallest = lmi->heightPref;
		}
	}
	if (flags & SAMESIZE) 
	{//SAMESIZE makes all items the same size as the largest
		index = 0;
		//Resize all items to match largest:
		for (uint32 i = 0; i < rows; i++) 
		{
			for (uint32 j = 0; j < columns; j++) 
			{
				LayoutMatrixItem * lmi = (LayoutMatrixItem *)itemsList->ItemAt(index++);
				switch (lmi->kind)
				{
					case KIND_MYBUTTON:
					{
						BButton * scratchMyButton = (BButton *)lmi->item;
						view.AddChild(scratchMyButton);
						scratchMyButton->ResizeTo(widest, tallest);
						view.RemoveChild(scratchMyButton);
					break;
					}
					case KIND_STRINGDRAWER:
					{
						BStringView * scratchMyStringView = (BStringView *)lmi->item;
						view.AddChild(scratchMyStringView);
						scratchMyStringView->ResizeTo(widest, tallest);
							view.RemoveChild(scratchMyStringView);
					break;
					}
					case KIND_MYCHECKBOX:
					{
						BCheckBox * scratchMyCheckBox = (BCheckBox *)lmi->item;
						view.AddChild(scratchMyCheckBox);
						scratchMyCheckBox->ResizeTo(widest, tallest);
						view.RemoveChild(scratchMyCheckBox);
					break;
					}
					case KIND_MYPICTUREBUTTON:
					{
						BPictureButton * scratchMyPictureButton = (BPictureButton *)lmi->item;
						view.AddChild(scratchMyPictureButton);
						scratchMyPictureButton->ResizeTo(widest, tallest);
						view.RemoveChild(scratchMyPictureButton);
					break;
					}
					case KIND_MYCOLORCONTROL:
					{
						BColorControl * scratchMyColorControl = (BColorControl *)lmi->item;
						view.AddChild(scratchMyColorControl);
						scratchMyColorControl->ResizeTo(widest, tallest);
						view.RemoveChild(scratchMyColorControl);
					break;
					}
					case KIND_MYSTATUSBAR:
					{
						BStatusBar * scratchMyStatusBar = (BStatusBar *)lmi->item;
						view.AddChild(scratchMyStatusBar);
						scratchMyStatusBar->ResizeTo(widest, tallest);
						view.RemoveChild(scratchMyStatusBar);
					break;
					}
					case KIND_INTGADG:
					{
						IntGadget * scratchIntGadget = (IntGadget *)lmi->item;
						view.AddChild(scratchIntGadget);
						scratchIntGadget->ResizeTo(widest, tallest);
						view.RemoveChild(scratchIntGadget);
					break;
					}
					
					case KIND_STRINGGADG:
					{
						StringGadget * scratchStringGadget = (StringGadget *)lmi->item;
						view.AddChild(scratchStringGadget);
						scratchStringGadget->ResizeTo(widest, tallest);
						view.RemoveChild(scratchStringGadget);
					break;
					}
//					case KIND_TEXT: //nothing needed from this
//					case KIND_COLORWELLGADG: //debug work needed here??
				}
			}
		}
	}
	index = 0;
	//move every item based upon size and padding:
	const float hpad = 2;	//horizontal spacing between items
	const float vpad = 2;	//vertical spacing between items
	float localLeft;
	float localTop = vpad + fparamMatrixTop;
	float excessRight = 0, excessRightHold;
	float excessBottom = 0, excessBottomHold;
	for (uint32 i = 0; i < rows; i++) 
	{
		localLeft = fparamMatrixLeft + hpad;
		float columnRight = 0;
		for (uint32 j = 0; j < columns; j++) 
		{
			LayoutMatrixItem * lmi = (LayoutMatrixItem *)itemsList->ItemAt(index++);
			//debug below we need to consider those items not resized under SAME
			switch (lmi->kind) 
			{
				case KIND_MYPOPUPMENU:
				{
					BPopUpMenu * scratchPopUpMenu = (BPopUpMenu *)lmi->item;
					scratchPopUpMenu->MoveTo(localLeft, localTop);
					excessRightHold = ((MyPopUpMenu *)scratchPopUpMenu)->fmWidest;
					excessBottomHold = ((MyPopUpMenu *)scratchPopUpMenu)->fmHeight;
				break;
				}
				case KIND_MYTEXTVIEW:
				{
					BTextView * scratchTextView = (BTextView *)lmi->item;
					scratchTextView->MoveTo(localLeft, localTop);
					excessRightHold = ((MyTextView *)scratchTextView)->fmWidth;
					excessBottomHold = ((MyTextView *)scratchTextView)->fmHeight;
				break;
				}
				case KIND_MYBUTTON:
				{
					BButton * scratchMyButton = (BButton *)lmi->item;
					view.AddChild(scratchMyButton);
					scratchMyButton->MoveTo(localLeft, localTop);
					excessRightHold = scratchMyButton->Bounds().Width();
					excessBottomHold = scratchMyButton->Bounds().Height();
					view.RemoveChild(scratchMyButton);
				break;
				}
				case KIND_MYRADIOVIEW:
				{
					MyRadioView * scratchRadioView = (MyRadioView *)lmi->item;
					view.AddChild(scratchRadioView);
					scratchRadioView->MoveTo(localLeft, localTop);
					excessRightHold = scratchRadioView->Bounds().Width();
					excessBottomHold = scratchRadioView->Bounds().Height();
					view.RemoveChild(scratchRadioView);
				break;
				}
				case KIND_STRINGDRAWER:
				{
					BStringView * scratchMyStringView = (BStringView *)lmi->item;
					view.AddChild(scratchMyStringView);
					TextItem * scratchTextItem = ((MyStringDrawer *)lmi->item)->GetTextItem();
					scratchTextItem->left = localLeft;
					scratchTextItem->top = localTop + lmi->heightPref - ((MyStringDrawer *)lmi->item)->descender - vpad;
					scratchMyStringView->MoveTo(localLeft, localTop);
					excessRightHold = scratchMyStringView->Bounds().Width();
					lmi->widthPref = excessRightHold;
					excessBottomHold = scratchMyStringView->Bounds().Height();
					lmi->heightPref = excessBottomHold;
					view.RemoveChild(scratchMyStringView);
				break;
				}
				case KIND_MYCHECKBOX:
				{
					BCheckBox * scratchMyCheckBox = (BCheckBox *)lmi->item;
					view.AddChild(scratchMyCheckBox);
					scratchMyCheckBox->MoveTo(localLeft, localTop);
					excessRightHold = scratchMyCheckBox->Bounds().Width();
					excessBottomHold = scratchMyCheckBox->Bounds().Height();
					view.RemoveChild(scratchMyCheckBox);
					if (lmi->last)
					{//there must be a math/logic error somewhere which requires this extra spacing
						excessBottomHold += (2 * MATRIXVERTICALOFFSETGUTS);
					}
				break;
				}
				case KIND_MYPICTUREBUTTON:
				{
					BPictureButton * scratchMyPictureButton = (BPictureButton *)lmi->item;
					view.AddChild(scratchMyPictureButton);
					scratchMyPictureButton->MoveTo(localLeft, localTop);
					excessRightHold = scratchMyPictureButton->Bounds().Width();
					excessBottomHold = scratchMyPictureButton->Bounds().Height();
					view.RemoveChild(scratchMyPictureButton);
				break;
				}
				case KIND_MYCOLORCONTROL:
				{
					BColorControl * scratchMyColorControl = (BColorControl *)lmi->item;
					view.AddChild(scratchMyColorControl);
					scratchMyColorControl->MoveTo(localLeft, localTop);
					excessRightHold = scratchMyColorControl->Bounds().Width();
					excessBottomHold = scratchMyColorControl->Bounds().Height();
					if (lmi->last)
					{//there must be a math/logic error somewhere which requires this extra spacing
						excessBottomHold += MATRIXVERTICALOFFSETGUTS;
					}
					view.RemoveChild(scratchMyColorControl);
				break;
				}
				case KIND_MYSTATUSBAR:
				{
					BStatusBar * scratchMyStatusBar = (BStatusBar *)lmi->item;
					view.AddChild(scratchMyStatusBar);
					scratchMyStatusBar->MoveTo(localLeft, localTop);
					excessRightHold = scratchMyStatusBar->Bounds().Width();
					excessBottomHold = scratchMyStatusBar->Bounds().Height();
					view.RemoveChild(scratchMyStatusBar);
				break;
				}
				case KIND_INTGADG:
				{
					IntGadget * scratchIntGadget = (IntGadget *)lmi->item;
					view.AddChild(scratchIntGadget);
					scratchIntGadget->MoveTo(localLeft, localTop);
					excessRightHold = scratchIntGadget->Bounds().Width();
					excessBottomHold = scratchIntGadget->Bounds().Height();
					view.RemoveChild(scratchIntGadget);
				break;
				}
				
				case KIND_STRINGGADG:
				{
					StringGadget * scratchStringGadget = (StringGadget *)lmi->item;
					view.AddChild(scratchStringGadget);
					scratchStringGadget->MoveTo(localLeft, localTop);
					excessRightHold = scratchStringGadget->Bounds().Width();
					excessBottomHold = scratchStringGadget->Bounds().Height();
					view.RemoveChild(scratchStringGadget);
				break;
				}
				case KIND_TEXT:
					scratchTextItem = (TextItem *)lmi->item;
					scratchTextItem->left = localLeft;
					scratchTextItem->top = localTop + lmi->heightPref;
					excessRightHold = lmi->widthPref;
					excessBottomHold = lmi->heightPref;
				break;
				case KIND_COLORWELLGADG:
					scratchTextItem = ((ColorWell *)lmi->item)->GetTextItem();
					scratchTextItem->left = localLeft;
					((ColorWell *)lmi->item)->SetLeft(((ColorWell *)lmi->item)->GetLeft() + localLeft);
					excessRightHold = lmi->widthPref;
					scratchTextItem->top = localTop + lmi->heightPref;
					((ColorWell *)lmi->item)->SetTop(((ColorWell *)(lmi->item))->descender + (scratchTextItem->top - ((((ColorWell *)lmi->item)->GetTop()) / 2)) - (lmi->heightPref / 2));
					excessBottomHold = lmi->heightPref + ((ColorWell *)(lmi->item))->descender;
				break;
			}
			excessRightHold += hpad;
			if (flags & SAMESIZE)
			{
				if (excessRightHold > excessRight)
				{
					excessRight = excessRightHold;
				}
			}
			else
			{
				excessRight = excessRightHold;
			}
			localLeft += excessRight;
			if (columns - 1) 
			{
				if (j != (columns - 1)) 
				{
					if (localLeft > columnRight) 
					{
						columnRight = localLeft;
					}
				}
				else 
				{
					columnRight = localLeft;
				}
			}
			else 
			{
				if (localLeft > columnRight) 
				{
					columnRight = localLeft;
				}
			}
			//right is stored so that another MatrixLayout
			//can know where a previous one ended up
		}
		if (columnRight > right)
		{
			right = columnRight;
		}
		excessBottomHold += vpad;
		if (flags & SAMESIZE)
		{
			if (excessBottomHold > excessBottom)
			{
				excessBottom = excessBottomHold;
			}
		}
		else
		{
			excessBottom = excessBottomHold;
		}
		localTop += excessBottom;
		if (rows - 1) 
		{
			if (i != (rows - 1)) 
			{
				if (localTop > bottom) 
				{
					bottom = localTop;
				}
			}
			else 
			{
				bottom = localTop;
			}
		}
		else 
		{
			if (localTop > bottom) 
			{
				bottom = localTop;
			}
		}
		//bottom is stored so that another MatrixLayout
		//can know where a previous one ended up
	}
	window->RemoveChild(&view);
	window->Unlock();
	release_sem(paramSemID);
}//end


void
LayoutMatrix	::	AddToChildren(	void * item) 
{
	mpChildrenToAdd->AddItem(item);
}//end


void
LayoutMatrix	::	AddToDraw(	void * item) 
{
	mpItemsToDraw->AddItem(item);
}//end